#include <stdio.h>
#include <fcntl.h>
#include <linux/types.h>
#include <linux/kd.h>
#include <linux/keyboard.h>
#include "ksyms.h"

static int fd;

void print_binding(char * line, unsigned char index, unsigned char table)
{
	struct kbentry ke;
	int t;
	int v;
	char *p;

	line = line+strlen(line);
	ke.kb_index = index;
	ke.kb_table = table;
	if (ioctl(fd, KDGKBENT, (unsigned long)&ke)) {
		fprintf(stderr, "KDGKBENT at index %d in table %d", index, table);
		perror("");
		exit(1);
	}
	t = KTYP(ke.kb_value);
	v = KVAL(ke.kb_value);
	if (t < syms_size && v < syms[t].size && (p = syms[t].table[v])[0])
		sprintf(line, "%-16s ", p);
	else
		sprintf(line, "0x%04x           ", ke.kb_value);
}

void dump_symbols(void)
{
	int t;
	int v;
	char *p;

	printf("# symbols recognized by dumpkeys\n");
	for (t = 0; t < syms_size; t++)
		for (v = 0; v < syms[t].size; v++)
			if ((p = syms[t].table[v])[0])
				printf("%s\n", p);
}

void usage(void)
{
	fprintf(stderr, "usage: dumpkeys [-s]\n");
	exit(1);
}

main (int argc, char *argv[])
{
	int i;
	int j;
	int c;
	char line[256];
	char dump = 0;

	extern int optind;

	while ((c = getopt(argc, argv, "s")) != -1) {
		switch (c) {
			case 's':
				dump = 1;
				break;
			case '?':
				usage();
		}
	}

	if (optind < argc)
		usage();

	if (dump) {
		dump_symbols();
		exit(0);
	}

	if ((fd = open("/dev/console", O_RDONLY)) < 0) {
		perror("when opening /dev/console");
		exit(1);
	}

	for (i = 0; i < NR_KEYS; i++) {
		sprintf(line,"keycode %3d = ", i);
		for (j = 0; j < NR_KEYMAPS; j++)
			print_binding(line, i, j);
		j = strlen(line);
		for (;;) {
			while (j-- > 0)
				if (line[j] != ' ')
					break;
				else
					line[j] = '\0';
			j -= strlen("VoidSymbol")-1;
			if (j < 0)
				break;
			if (strcmp("VoidSymbol",line+j))
				break;
			line[j] = '\0';
		}
		puts(line);
	}
	exit(0);
}
