%token EOL NUMBER LITERAL KEYCODE EQUALS

%{
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <linux/kd.h>
#include "analyze.c"

static void defkey(int, int, int);
int voidkey;
%}

%%
keytable	:
		| keytable line
		;
line		: EOL
		| KEYCODE NUMBER EQUALS EOL
			{
			defkey($2,K_NORMTAB,voidkey);
			defkey($2,K_SHIFTTAB,voidkey);
			defkey($2,K_ALTTAB,voidkey);
			}
		| KEYCODE NUMBER EQUALS rvalue EOL
			{
			defkey($2,K_NORMTAB,$4);
			defkey($2,K_SHIFTTAB,voidkey);
			defkey($2,K_ALTTAB,voidkey);
			}
		| KEYCODE NUMBER EQUALS rvalue rvalue EOL
			{
			defkey($2,K_NORMTAB,$4);
			defkey($2,K_SHIFTTAB,$5);
			defkey($2,K_ALTTAB,voidkey);
			}
		| KEYCODE NUMBER EQUALS rvalue rvalue rvalue EOL
			{
			defkey($2,K_NORMTAB,$4);
			defkey($2,K_SHIFTTAB,$5);
			defkey($2,K_ALTTAB,$6);
			}
		;
rvalue		: NUMBER
			{$$=$1;}
		| LITERAL
			{$$=$1;}
		;
%%			

int fd;

int main(argc,argv)
unsigned int    argc;
char            **argv;
{

	voidkey = ksymtocode("VoidSymbol");
	if (argc>2) {
		fprintf(stderr,"usage: %s [layout_file]\n", argv[0]);
		exit(1);
	}
	if (argc==2) {
		if (freopen(argv[1], "r", stdin) == NULL) {
			fprintf(stderr, "cannot open file %s\n", argv[1]);
			exit(1);
		}
	}
	if ((fd = open("/dev/console", O_RDONLY)) < 0) {
		fprintf(stderr, "cannot open console for ioctl\n");
		exit(1);
	}
	if (yyparse()) {
		fprintf(stderr,"syntax error in layout file\n");
		exit(1);
	}
	exit(0);
}

yyerror(char *s)
{
	fprintf(stderr, "line %d: %s\n", line_nr, s);
}

static void defkey(int index, int table, int keycode)
{
	struct kbentry ke;

	if (keycode == -1)
		return;

	ke.kb_index = index;
	ke.kb_table = table;
	ke.kb_value = keycode;

	if (ioctl(fd, KDSKBENT, (unsigned long)&ke))
		fprintf(stderr, "failed to bind key %d to value %d\n",
			index, keycode);
}
